import React, { HTMLAttributes, PropsWithChildren, useCallback, useEffect, useRef, useState } from 'react';
import { Autocomplete, AutocompleteRenderGroupParams, List, AutocompleteRenderInputParams } from '@mui/material';
import { NsTypography, NsButton } from '../';
import { NsAutocompleteProps, OptionsProps, ComponentBoxProps } from './types.d';
import { StyledTextField } from './NsAutocomplete.styled';

const renderOptionLabel = (option: OptionsProps) => option.value;

const renderGroupBy = (option: OptionsProps) => option.group as string;

const NsAutocomplete = ({
    id = 'custom_autocomplete_dropdown',
    options,
    OptionsUI,
    loading,
    noOptionsText,
    onSelect,
    onChange,
    clear,
    seeMore,
    placeholder,
    className,
    iconStart,
    iconEnd,
    inputValue,
    value,
}: PropsWithChildren<NsAutocompleteProps>) => {
    const [scrollTopPosition, setScrollTopPosition] = useState(0);
    const listboxRef = useRef<HTMLUListElement | null>(null);

    useEffect(() => {
        if (listboxRef?.current && options.length > 0) {
            listboxRef.current.scrollTop = scrollTopPosition;
        }
    }, [options, scrollTopPosition]);

    const handleSelect = useCallback(
        (event: React.SyntheticEvent, value: string | OptionsProps | null) => {
            if (typeof onSelect === 'function') {
                onSelect(value);
            }
        },
        [onSelect]
    );

    const handleInputChange = useCallback(
        (event: React.SyntheticEvent, newInputValue: string) => {
            if (typeof onChange === 'function') {
                onChange(newInputValue);
            }
        },
        [onChange]
    );

    const renderInput = (params: AutocompleteRenderInputParams) => (
        <StyledTextField
            {...params}
            placeholder={placeholder}
            InputProps={{
                ...params.InputProps,
                startAdornment: iconStart,
                endAdornment: iconEnd,
            }}
        />
    );

    // TODO: @vijayprathap need to check if we can remove object spread of ...props and ...params in renderInput
    const renderOption = (props: ComponentBoxProps, option: OptionsProps) => (
        <li {...props} key={props.id}>
            {OptionsUI ? <OptionsUI {...props} option={option} /> : null}
        </li>
    );

    const renderGroupName = ({ key, group, children }: AutocompleteRenderGroupParams) => (
        <li key={key}>
            {group ? (
                <NsTypography variant="p-m" weight="bold" className="p-3">
                    {group}
                </NsTypography>
            ) : null}
            <ul className="p-0">{children}</ul>
        </li>
    );

    const renderClearButton = useCallback(() => {
        const { show, onClear, text = 'Clear Search' } = clear || {};
        if (!show) {
            return null;
        }
        return (
            <li className="text-right p-3">
                <NsButton onClick={onClear} data-testid="autocomplete-clear-id">
                    <NsTypography variant="p-xs" weight="bold">
                        {text}
                    </NsTypography>
                </NsButton>
            </li>
        );
    }, [clear]);

    const handleSeeMore = useCallback(() => {
        const { onSeeMore } = seeMore || {};
        // Remember the current scroll position
        if (options?.length > 0 && listboxRef?.current) {
            const lastScrollPosition = listboxRef.current?.scrollTop;
            if (lastScrollPosition) {
                setScrollTopPosition(lastScrollPosition);
            }
        }
        if (onSeeMore && typeof onSeeMore === 'function') {
            onSeeMore();
        }
    }, [seeMore, options]);

    const renderSeeMoreButton = useCallback(() => {
        const { show, text = 'See More Results' } = seeMore || {};
        if (!show) {
            return null;
        }

        return (
            <li className="text-center p-3">
                <NsButton onClick={handleSeeMore} data-testid="autocomplete-loadmore-id">
                    <NsTypography variant="p-xs" weight="bold">
                        {text}
                    </NsTypography>
                </NsButton>
            </li>
        );
    }, [seeMore, handleSeeMore]);

    const renderListBoxComponent = ({ children, ...otherProps }: ComponentBoxProps) => (
        <List {...otherProps} ref={listboxRef}>
            {renderClearButton()}
            {children}
            {renderSeeMoreButton()}
        </List>
    );

    return (
        <Autocomplete
            id={id}
            disablePortal
            className={className}
            options={options}
            loading={loading}
            noOptionsText={noOptionsText}
            groupBy={renderGroupBy}
            getOptionLabel={renderOptionLabel}
            renderOption={renderOption}
            renderInput={renderInput}
            ListboxComponent={renderListBoxComponent}
            ListboxProps={{ scrollTopPosition } as HTMLAttributes<HTMLUListElement>} // Pass the current scroll position
            renderGroup={renderGroupName}
            onChange={handleSelect}
            onInputChange={handleInputChange}
            value={value}
            inputValue={inputValue}
            freeSolo
        />
    );
};

export default NsAutocomplete;
